<?php
/**
 * Plugin Name: WPC Free Shipping Bar for WooCommerce
 * Plugin URI: https://wpclever.net/
 * Description: Encourage customers to increase their order value to be qualified for free shipping with a beautiful customizable bar.
 * Version: 1.0.3
 * Author: WPClever
 * Author URI: https://wpclever.net
 * Text Domain: wpc-free-shipping-bar
 * Domain Path: /languages/
 * Requires at least: 4.0
 * Tested up to: 5.9
 * WC requires at least: 3.0
 * WC tested up to: 6.3
 */

defined( 'ABSPATH' ) || exit;

! defined( 'WPCFB_VERSION' ) && define( 'WPCFB_VERSION', '1.0.3' );
! defined( 'WPCFB_URI' ) && define( 'WPCFB_URI', plugin_dir_url( __FILE__ ) );
! defined( 'WPCFB_REVIEWS' ) && define( 'WPCFB_REVIEWS', 'https://wordpress.org/support/plugin/wpc-free-shipping-bar/reviews/?filter=5' );
! defined( 'WPCFB_CHANGELOG' ) && define( 'WPCFB_CHANGELOG', 'https://wordpress.org/plugins/wpc-free-shipping-bar/#developers' );
! defined( 'WPCFB_DISCUSSION' ) && define( 'WPCFB_DISCUSSION', 'https://wordpress.org/support/plugin/wpc-free-shipping-bar' );
! defined( 'WPC_URI' ) && define( 'WPC_URI', WPCFB_URI );

include 'includes/wpc-dashboard.php';
include 'includes/wpc-menu.php';
include 'includes/wpc-kit.php';
include 'includes/wpc-notice.php';

if ( ! function_exists( 'wpcfb_init' ) ) {
	add_action( 'plugins_loaded', 'wpcfb_init', 11 );

	if ( ! function_exists( 'WC' ) || ! version_compare( WC()->version, '3.0', '>=' ) ) {
		add_action( 'admin_notices', 'wpcfb_notice_wc' );

		return;
	}

	function wpcfb_init() {
		// load text-domain
		load_plugin_textdomain( 'wpc-free-shipping-bar', false, basename( __DIR__ ) . '/languages/' );

		if ( ! class_exists( 'WPCleverWpcfb' ) && class_exists( 'WC_Product' ) ) {
			class WPCleverWpcfb {
				protected static $instance;
				protected static $localization = array();

				public static function instance() {
					if ( ! isset( self::$instance ) ) {
						self::$instance = new self();
						self::$instance->start();
					}

					return self::$instance;
				}

				function start() {
					add_action( 'init', array( $this, 'init' ) );
					add_action( 'admin_menu', array( $this, 'admin_menu' ) );
					add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 99 );
					add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );

					// settings link
					add_filter( 'plugin_action_links', array( $this, 'action_links' ), 10, 2 );
					add_filter( 'plugin_row_meta', array( $this, 'row_meta' ), 10, 2 );

					if ( get_option( 'wpcfb_show_mini_cart', 'yes' ) === 'yes' ) {
						// mini-cart
						add_action( 'woocommerce_widget_shopping_cart_before_buttons', array(
							$this,
							'free_shipping_bar'
						) );
					}

					if ( get_option( 'wpcfb_show_cart', 'yes' ) === 'yes' ) {
						// cart page
						add_action( 'woocommerce_proceed_to_checkout', array( $this, 'free_shipping_bar' ), 10 );
					}

					if ( get_option( 'wpcfb_show_checkout', 'yes' ) === 'yes' ) {
						// checkout page
						add_action( 'woocommerce_review_order_before_submit', array( $this, 'free_shipping_bar' ), 10 );
					}
				}

				function init() {
					self::$localization = (array) get_option( 'wpcfb_localization' );
					add_shortcode( 'wpcfb', array( $this, 'shortcode' ) );
				}

				function enqueue_scripts() {
					wp_enqueue_style( 'wpcfb-frontend', WPCFB_URI . 'assets/css/frontend.css', false, WPCFB_VERSION );
				}

				function admin_enqueue_scripts( $hook ) {
					if ( strpos( $hook, 'wpcfb' ) ) {
						wp_enqueue_style( 'wp-color-picker' );
						wp_enqueue_script( 'wpcfb-backend', WPCFB_URI . 'assets/js/backend.js', array(
							'jquery',
							'wp-color-picker'
						), WPCFB_VERSION );
					}
				}

				function shortcode() {
					ob_start();
					$this->free_shipping_bar();

					return ob_get_clean();
				}

				function admin_menu() {
					add_submenu_page( 'wpclever', esc_html__( 'WPC Free Shipping Bar', 'wpc-free-shipping-bar' ), esc_html__( 'Free Shipping Bar', 'wpc-free-shipping-bar' ), 'manage_options', 'wpclever-wpcfb', array(
						$this,
						'settings_page'
					) );
				}

				function settings_page() {
					add_thickbox();
					$active_tab = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : 'settings';
					?>
                    <div class="wpclever_settings_page wrap">
                        <h1 class="wpclever_settings_page_title"><?php echo 'WPC Free Shipping Bar ' . WPCFB_VERSION; ?></h1>
                        <div class="wpclever_settings_page_desc about-text">
                            <p>
								<?php printf( esc_html__( 'Thank you for using our plugin! If you are satisfied, please reward it a full five-star %s rating.', 'wpc-free-shipping-bar' ), '<span style="color:#ffb900">&#9733;&#9733;&#9733;&#9733;&#9733;</span>' ); ?>
                                <br/>
                                <a href="<?php echo esc_url( WPCFB_REVIEWS ); ?>"
                                   target="_blank"><?php esc_html_e( 'Reviews', 'wpc-free-shipping-bar' ); ?></a> | <a
                                        href="<?php echo esc_url( WPCFB_CHANGELOG ); ?>"
                                        target="_blank"><?php esc_html_e( 'Changelog', 'wpc-free-shipping-bar' ); ?></a>
                                | <a href="<?php echo esc_url( WPCFB_DISCUSSION ); ?>"
                                     target="_blank"><?php esc_html_e( 'Discussion', 'wpc-free-shipping-bar' ); ?></a>
                            </p>
                        </div>
                        <div class="wpclever_settings_page_nav">
                            <h2 class="nav-tab-wrapper">
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpcfb&tab=settings' ); ?>"
                                   class="<?php echo $active_tab === 'settings' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Settings', 'wpc-free-shipping-bar' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpcfb&tab=localization' ); ?>"
                                   class="<?php echo $active_tab === 'localization' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Localization', 'wpc-free-shipping-bar' ); ?>
                                </a>
                                <a href="<?php echo esc_url( admin_url( 'admin.php?page=wpclever-kit' ) ); ?>"
                                   class="nav-tab">
									<?php esc_html_e( 'Essential Kit', 'wpc-free-shipping-bar' ); ?>
                                </a>
                            </h2>
                        </div>
                        <div class="wpclever_settings_page_content">
							<?php if ( $active_tab === 'settings' ) { ?>
                                <form method="post" action="options.php">
									<?php wp_nonce_field( 'update-options' ) ?>
                                    <table class="form-table">
                                        <tr class="heading">
                                            <th colspan="2">
												<?php esc_html_e( 'General', 'wpc-free-shipping-bar' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Show on mini-cart widget', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_show_mini_cart">
                                                    <option value="yes" <?php echo( get_option( 'wpcfb_show_mini_cart', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo( get_option( 'wpcfb_show_mini_cart', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Show on cart page', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_show_cart">
                                                    <option value="yes" <?php echo( get_option( 'wpcfb_show_cart', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo( get_option( 'wpcfb_show_cart', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Show on checkout page', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_show_checkout">
                                                    <option value="yes" <?php echo( get_option( 'wpcfb_show_checkout', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo( get_option( 'wpcfb_show_checkout', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Shortcode', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
												<?php printf( esc_html__( 'You can use shortcode %s to show the free shipping bar wherever you want.', 'wpc-free-shipping-bar' ), '<code>[wpcfb]</code>' ); ?>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Show qualified message', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_show_qualified">
                                                    <option value="yes" <?php echo( get_option( 'wpcfb_show_qualified', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo( get_option( 'wpcfb_show_qualified', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description">
											<?php esc_html_e( 'Show qualified message when reaching the free shipping amount.', 'wpc-free-shipping-bar' ); ?>
										</span>
                                            </td>
                                        </tr>
                                        <tr class="heading">
                                            <th colspan="2">
												<?php esc_html_e( 'Design', 'wpc-free-shipping-bar' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Style', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_style">
                                                    <option value="square" <?php echo( get_option( 'wpcfb_style', 'square' ) === 'square' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Square', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="rounded" <?php echo( get_option( 'wpcfb_style', 'square' ) === 'rounded' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Rounded', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Bar color', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
												<?php $wpcfb_bar_color_default = apply_filters( 'wpcfb_bar_color_default', '#ecd4e5' ); ?>
                                                <input type="text" name="wpcfb_bar_color"
                                                       value="<?php echo get_option( 'wpcfb_bar_color', $wpcfb_bar_color_default ); ?>"
                                                       class="wpcfb_color_picker"/>
                                                <span class="description">
											<?php printf( esc_html__( 'Choose the background color for the bar, default %s', 'wpc-free-shipping-bar' ), '<code>' . $wpcfb_bar_color_default . '</code>' ); ?>
										</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Progress color', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
												<?php $wpcfb_progress_color_default = apply_filters( 'wpcfb_progress_color_default', '#95578a' ); ?>
                                                <input type="text" name="wpcfb_progress_color"
                                                       value="<?php echo get_option( 'wpcfb_progress_color', $wpcfb_progress_color_default ); ?>"
                                                       class="wpcfb_color_picker"/>
                                                <span class="description">
											<?php printf( esc_html__( 'Choose the background color for the progress, default %s', 'wpc-free-shipping-bar' ), '<code>' . $wpcfb_progress_color_default . '</code>' ); ?>
										</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th scope="row"><?php esc_html_e( 'Animated', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <select name="wpcfb_progress_animated">
                                                    <option value="yes" <?php echo( get_option( 'wpcfb_progress_animated', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo( get_option( 'wpcfb_progress_animated', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-free-shipping-bar' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description">
											<?php esc_html_e( 'Add animation for progress bar.', 'wpc-free-shipping-bar' ); ?>
										</span>
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
                                                <input type="submit" name="submit" class="button button-primary"
                                                       value="<?php esc_html_e( 'Update Options', 'wpc-free-shipping-bar' ); ?>"/>
                                                <input type="hidden" name="action" value="update"/>
                                                <input type="hidden" name="page_options"
                                                       value="wpcfb_show_mini_cart,wpcfb_show_cart,wpcfb_show_checkout,wpcfb_show_qualified,wpcfb_style,wpcfb_bar_color,wpcfb_progress_color,wpcfb_progress_animated"/>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } elseif ( $active_tab === 'localization' ) { ?>
                                <form method="post" action="options.php">
									<?php wp_nonce_field( 'update-options' ) ?>
                                    <table class="form-table">
                                        <tr class="heading">
                                            <th scope="row"><?php esc_html_e( 'Localization', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
												<?php esc_html_e( 'Leave blank to use the default text and its equivalent translation in multiple languages.', 'wpc-free-shipping-bar' ); ?>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Title', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <input type="text" name="wpcfb_localization[title]" style="width: 100%"
                                                       value="<?php echo esc_attr( self::localization( 'title' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( 'Free delivery on orders over {free_shipping_amount}', 'wpc-free-shipping-bar' ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Message', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <input type="text" name="wpcfb_localization[message]"
                                                       style="width: 100%"
                                                       value="<?php echo esc_attr( self::localization( 'message' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( 'Add at least {remaining} more to enjoy the free shipping!', 'wpc-free-shipping-bar' ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Qualified message', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
                                                <input type="text" name="wpcfb_localization[qualified]"
                                                       style="width: 100%"
                                                       value="<?php echo esc_attr( self::localization( 'qualified' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( 'Your order is qualified for free shipping!', 'wpc-free-shipping-bar' ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Placeholder', 'wpc-free-shipping-bar' ); ?></th>
                                            <td>
												<?php esc_html_e( '{free_shipping_amount}: free shipping amount', 'wpc-free-shipping-bar' ); ?>
                                                <br/>
												<?php esc_html_e( '{remaining}: remaining amount', 'wpc-free-shipping-bar' ); ?>
                                                <br/>
												<?php esc_html_e( '{subtotal}: cart subtotal', 'wpc-free-shipping-bar' ); ?>
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
                                                <input type="submit" name="submit" class="button button-primary"
                                                       value="<?php esc_attr_e( 'Update Options', 'wpc-free-shipping-bar' ); ?>"/>
                                                <input type="hidden" name="action" value="update"/>
                                                <input type="hidden" name="page_options" value="wpcfb_localization"/>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } ?>
                        </div>
                    </div>
					<?php
				}

				function action_links( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$settings = '<a href="' . admin_url( 'admin.php?page=wpclever-wpcfb&tab=settings' ) . '">' . esc_html__( 'Settings', 'wpc-free-shipping-bar' ) . '</a>';
						array_unshift( $links, $settings );
					}

					return (array) $links;
				}

				function row_meta( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$row_meta = array(
							'support' => '<a href="' . esc_url( WPCFB_DISCUSSION ) . '" target="_blank">' . esc_html__( 'Community support', 'wpc-free-shipping-bar' ) . '</a>',
						);

						return array_merge( $links, $row_meta );
					}

					return (array) $links;
				}

				public function free_shipping_bar() {
					$amount_for_free_shipping = $this->get_free_shipping_min_amount();

					if ( ! $amount_for_free_shipping ) {
						return;
					}

					$show_shipping_before_address = WC()->cart->show_shipping();

					if ( ! $show_shipping_before_address ) {
						return;
					}

					$cart_total             = WC()->cart->get_displayed_subtotal();
					$discount               = WC()->cart->get_discount_total();
					$discount_tax           = WC()->cart->get_discount_tax();
					$price_including_tax    = WC()->cart->display_prices_including_tax();
					$price_decimal          = wc_get_price_decimals();
					$ignore_coupon          = apply_filters( 'wpcfb_ignore_coupon', true );
					$local_pickup           = apply_filters( 'wpcfb_local_pickup', true );
					$show_qualified_message = get_option( 'wpcfb_show_qualified', 'yes' ) === 'yes';
					$title                  = self::localization( 'title', esc_html__( 'Free delivery on orders over {free_shipping_amount}', 'wpc-free-shipping-bar' ) );
					$message                = self::localization( 'message', esc_html__( 'Add at least {remaining} more to enjoy the free shipping!', 'wpc-free-shipping-bar' ) );
					$qualified              = self::localization( 'qualified', esc_html__( 'Your order is qualified for free shipping!', 'wpc-free-shipping-bar' ) );
					$bar_color              = get_option( 'wpcfb_bar_color', apply_filters( 'wpcfb_bar_color_default', '#ecd4e5' ) );
					$progress_color         = get_option( 'wpcfb_progress_color', apply_filters( 'wpcfb_progress_color_default', '#95578a' ) );

					if ( ! $local_pickup && $this->is_shipping_method( $this->get_shipping_method(), 'local_pickup' ) ) {
						return;
					}

					if ( $ignore_coupon ) {
						$discount     = 0;
						$discount_tax = 0;
					}

					if ( $price_including_tax ) {
						$cart_total = round( $cart_total - ( $discount + $discount_tax ), $price_decimal );
					} else {
						$cart_total = round( $cart_total - $discount, $price_decimal );
					}

					$remaining = $amount_for_free_shipping - $cart_total;
					$percent   = 100 - ( $remaining / $amount_for_free_shipping ) * 100;

					$title     = $this->placeholders( $title, $remaining, $amount_for_free_shipping );
					$message   = $this->placeholders( $message, $remaining, $amount_for_free_shipping );
					$qualified = $this->placeholders( $qualified, $remaining, $amount_for_free_shipping );

					if ( $cart_total < $amount_for_free_shipping ) {
						?>
                        <div class="wpcfb-wrap wpc-free-shipping-bar <?php echo esc_attr( 'wpcfb-style-' . get_option( 'wpcfb_style', 'square' ) ); ?> <?php echo esc_attr( get_option( 'wpcfb_progress_animated', 'yes' ) === 'yes' ? 'wpcfb-progress-animated' : '' ); ?>">
                            <div class="wpcfb-title"><?php echo $this->kses( $title ); ?></div>
                            <div class="wpcfb-progress-bar"
                                 style="background-color:<?php echo esc_attr( $bar_color ); ?>">
                        <span class="wpcfb-progress-amount"
                              style="width:<?php echo esc_attr( $percent . '%' ); ?>; background-color:<?php echo esc_attr( $progress_color ); ?>"></span>
                            </div>
                            <div class="wpcfb-message"><?php echo $this->kses( $message ); ?></div>
                        </div>
					<?php } elseif ( $show_qualified_message ) { ?>
                        <div class="wpcfb-wrap wpc-free-shipping-bar wpcfb-qualified-message">
                            <div class="wpcfb-title"><?php echo $this->kses( $qualified ); ?></div>
                        </div>
						<?php
					}
				}

				public function kses( $text ) {
					return wp_kses( $text, array(
						'bdi'  => array(),
						'span' => array( 'class' => array() )
					) );
				}

				public function get_free_shipping_min_amount() {
					$amount             = null;
					$chosen_shipping_id = $this->get_shipping_method();

					if ( ! $chosen_shipping_id ) {
						return '';
					}

					$is_flexible_shipping = $this->is_shipping_method( $chosen_shipping_id, 'flexible_shipping' );
					$is_free_shipping     = $this->is_shipping_method( $chosen_shipping_id, 'free_shipping' );
					$is_local_pickup      = $this->is_shipping_method( $chosen_shipping_id, 'local_pickup' );
					$is_flat_rate         = $this->is_shipping_method( $chosen_shipping_id, 'flat_rate' );

					if ( $is_flexible_shipping ) {
						$option_name = 'woocommerce_' . str_replace( ':', '_', $chosen_shipping_id ) . '_settings';
						$option      = get_option( $option_name );
						$amount      = isset( $option['method_free_shipping'] ) ? $option['method_free_shipping'] : null;

						return $amount ?: $this->get_shipping_method_min_amount( $chosen_shipping_id );
					}

					$packages      = WC()->cart->get_shipping_packages();
					$package       = reset( $packages );
					$zone          = wc_get_shipping_zone( $package );
					$free_shipping = null;

					foreach ( $zone->get_shipping_methods( true ) as $method ) {
						if ( $method->id === 'free_shipping' ) {
							$instance      = isset( $method->instance_settings ) ? $method->instance_settings : null;
							$free_shipping = isset( $instance['min_amount'] ) ? $instance['min_amount'] : null;
						}
					}

					if ( $is_flat_rate || $is_free_shipping || $is_local_pickup ) {
						$amount = $free_shipping;
					}

					return $amount;
				}

				public function get_shipping_method() {
					$chosen_methods = WC()->session->get( 'chosen_shipping_methods' );

					if ( ! $chosen_methods ) {
						return null;
					}

					$chosen_shipping_id = $chosen_methods ? $chosen_methods[0] : '';

					return $chosen_shipping_id;
				}

				public function get_shipping_method_min_amount( $shipping_id ) {
					$packages = WC()->shipping->get_packages();
					$amount   = null;

					foreach ( $packages as $package ) {
						if ( isset( $package['rates'] ) && isset( $package['rates'][ $shipping_id ] ) ) {
							$rate = $package['rates'][ $shipping_id ];

							$meta = $rate->get_meta_data();

							if ( isset( $meta['_fs_method']['method_free_shipping'] ) ) {
								$amount = $meta['_fs_method']['method_free_shipping'] ?: null;
							}
						}
					}

					return $amount;
				}

				public function is_shipping_method( $string, $start_string ) {
					$len = strlen( $start_string );

					return ( substr( $string, 0, $len ) === $start_string );
				}

				public function placeholders( $input_string = '', $remaining = null, $amount_for_free_shipping = null ) {
					if ( $remaining ) {
						$input_string = str_replace( '{remaining}', wc_price( $remaining ), $input_string );
					}

					if ( $amount_for_free_shipping ) {
						$input_string = str_replace( '{free_shipping_amount}', wc_price( $amount_for_free_shipping ), $input_string );
					}

					$input_string = str_replace( '{subtotal}', WC()->cart->get_cart_subtotal(), $input_string );

					return $input_string;
				}

				function localization( $key = '', $default = '' ) {
					$str = '';

					if ( ! empty( $key ) && ! empty( self::$localization[ $key ] ) ) {
						$str = self::$localization[ $key ];
					} elseif ( ! empty( $default ) ) {
						$str = $default;
					}

					return apply_filters( 'wpcfb_localization_' . $key, $str );
				}
			}

			WPCleverWpcfb::instance();
		}
	}
}

if ( ! function_exists( 'wpcfb_notice_wc' ) ) {
	function wpcfb_notice_wc() {
		?>
        <div class="error">
            <p><strong>WPC Free Shipping Bar</strong> require WooCommerce version 3.0 or greater.</p>
        </div>
		<?php
	}
}